## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = FALSE,
  warning = FALSE
  )

## ----load_packages, message = FALSE, warning = FALSE--------------------------
# load required packages
library(clustSIGNAL)
library(distances)
library(cluster)
library(aricode)
library(dplyr)
library(ggplot2)
library(patchwork)
library(scattermore)
library(scater)

## ----embryo_data_prep---------------------------------------------------------
data(mEmbryo2)
# this will load me_expr and me_data objects, containing gene expression 
# logcounts and cell metadata, respectively, to your environment
# to create a SpatialExperiment object we need gene expression, cell metadata, 
# and cell locations
spe <- SpatialExperiment(assays = list(logcounts = me_expr), colData = me_data,
                         # spatialCoordsNames requires column names in 
                         # me_data that contain xy-coordinates of cells
                         spatialCoordsNames = c("X", "Y"))
spe

## ----embryo_data_columns------------------------------------------------------
spe |> colData() |> colnames() # to list the column names in the metadata

## ----ClustSIGNAL_singleRun----------------------------------------------------
set.seed(100) # for reproducibility
samples <- "sample_id" # column name containing sample names
# the simplest run of ClustSIGNAL requires data in a SpatialExperiment object,
# with column name for sample labels specified, along with the type
# of output to generate (clusters, neighbours, and/or final spe object).
res_emb <- clustSIGNAL(spe, samples, outputs = "a") 

## ----embryo_result_list-------------------------------------------------------
res_emb |> names() # listing the names of the outputs generated

## ----embryo_clusters_head-----------------------------------------------------
res_emb$clusters |> head() # cluster data frame has cell IDs and cluster labels

## ----embryo_final_spe---------------------------------------------------------
# for convenience with downstream analyses, we can replace the original spe
# object with the one generated by ClustSIGNAL. This does not lead to any loss 
# of information as ClustSIGNAL only adds information to the spe object.
spe <- res_emb$spe_final
spe 

## ----colors-------------------------------------------------------------------
colors <- c("#635547", "#8EC792", "#9e6762", "#FACB12", "#3F84AA", "#0F4A9C", 
            "#ff891c", "#EF5A9D", "#C594BF", "#DFCDE4", "#139992", "#65A83E", 
            "#8DB5CE", "#005579", "#C9EBFB", "#B51D8D", "#532C8A", "#8870ad", 
            "#cc7818", "#FBBE92", "#EF4E22", "#f9decf", "#c9a997", "#C72228", 
            "#f79083", "#F397C0", "#DABE99", "#c19f70", "#354E23", "#C3C388",
            "#647a4f", "#CDE088", "#f7f79e", "#F6BFCB", "#7F6874", "#989898", 
            "#1A1A1A", "#FFFFFF", "#e6e6e6", "#77441B", "#F90026", "#A10037", 
            "#DA5921", "#E1C239", "#9DD84A")

## ----embryo_spatialPlots------------------------------------------------------
# for plotting with scater R package, we need to add the spatial coordinates 
# to the reduced dimension section of the spe object
reducedDim(spe, "spatial") <- spatialCoords(spe)

# spatial plot
spt_clust <- scater::plotReducedDim(spe, colour_by = "ClustSIGNAL",
                                    # specify spatial low dimension
                                    dimred = "spatial", point_alpha = 1,
                                    point_size = 4, scattermore = TRUE) +
  ggtitle("A") +
  scale_color_manual(values = colors) +
  guides(colour = guide_legend(title = "Clusters",
                               override.aes = list(size = 5))) +
  theme(text = element_text(size = 12))

# we can also display the entropy distribution at cluster-level to see which 
# clusters have cells from homogeneous/heterogeneous space. For this, we
# reorder the clusters by their median entropy value 
df_met <- spe |> colData() %>% as.data.frame()

ct_ent <- df_met %>%
  group_by(as.character(ClustSIGNAL)) %>%
  summarise(mdEntropy = median(entropy))
cellOrder <- ct_ent$mdEntropy
names(cellOrder) <- ct_ent$`as.character(ClustSIGNAL)`
cellOrder <- sort(cellOrder)

df_met$ClustSIGNAL <- factor(df_met$ClustSIGNAL, levels = names(cellOrder))
col_ent <- colors[as.numeric(names(cellOrder))]
box_clust <- df_met %>%
  ggplot(aes(x = ClustSIGNAL, y = entropy, fill = ClustSIGNAL)) +
  geom_boxplot() +
  scale_fill_manual(values = col_ent) +
  ggtitle("B") +
  labs(x = "ClustSIGNAL clusters", y = "Entropy", name = "Clusters") +
  theme_classic() +
  theme(legend.position = "none",
        text = element_text(size = 12),
        axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))

spt_clust + box_clust + patchwork::plot_layout(guides = "collect", 
                                               widths = c(2, 3))

## ----embryo_clusterMetrics----------------------------------------------------
# to assess the accuracy of clustering, the cluster labels are often compared to
# prior annotations. Here we compare ClustSIGNAL cluster labels to annotations 
# available with this public data
spe |> colData() %>% 
  as.data.frame() %>%
  summarise(
    ARI = aricode::ARI(celltype_mapped_refined, ClustSIGNAL), # calculate ARI
    NMI = aricode::NMI(celltype_mapped_refined, ClustSIGNAL)) # calculate NMI

## ----embryo_entropyMetrics----------------------------------------------------
# we can assess the overall entropy distribution of the dataset
spe |> colData() %>% 
  as.data.frame() %>%
  summarise(min_Entropy = min(entropy),
            max_Entropy = max(entropy),
            mean_Entropy = mean(entropy))

## ----entropyPlots-------------------------------------------------------------
# we can also visualize the distribution and spread of the entropy values in 
# the sample(s)
hst_ent <- spe |> colData() %>% 
  as.data.frame() %>%
  ggplot(aes(entropy)) +
  geom_histogram(binwidth = 0.05) +
  ggtitle("A") +
  labs(x = "Entropy", y = "Number of regions") +
  theme_classic() +
  theme(text = element_text(size = 12))

spt_ent <- scater::plotReducedDim(spe, colour_by = "entropy",
                                    # specify spatial low dimension
                                    dimred = "spatial", point_alpha = 1,
                                    point_size = 4, scattermore = TRUE) +
  ggtitle("B") +
  scale_colour_gradient2("Entropy", low = "grey", high = "blue") +
  scale_size_continuous(range = c(0, max(spe$entropy))) +
  theme(text = element_text(size = 12))

hst_ent + spt_ent

## ----hypothal_data_prep-------------------------------------------------------
# this will load mh_expr and mh_data objects, containing gene expression 
# logcounts and cell metadata, respectively, to your environment
data(mHypothal)

# creates spe object using gene expression, cell metadata, and cell locations
spe2 <- SpatialExperiment(assays = list(logcounts = mh_expr), colData = mh_data,
                          # spatialCoordsNames requires column names in 
                          # mh_data that contain xy-coordinates of cells
                          spatialCoordsNames = c("X", "Y"))
spe2

## ----hypothal_data_columns----------------------------------------------------
spe2 |> colData() |> colnames() # to list the column names in the metadata

## ----ClustSIGNAL_multiRun-----------------------------------------------------
set.seed(110) # for reproducibility
samples <- "samples" # column name containing sample names
# ClustSIGNAL can be run on a dataset with multiple samples. As before, we need
# the SpatialExperiment object and column name of sample labels in the object. 
# The method can be run in parallel through the threads option. Here we use 
# thread = 4 to use 4 cores.
res_hyp <- clustSIGNAL(spe2, samples, threads = 4, outputs = "a")

## ----hypothal_final_spe-------------------------------------------------------
# for convenience with downstream analyses, we replace the original spe object 
# with the one generated by ClustSIGNAL
spe2 <- res_hyp$spe_final
spe2

## ----hypothal_samples---------------------------------------------------------
samplesList <- spe2[[samples]] |> unique() # get unique sample names
samplesList

## ----hypothal_clusterMetrics--------------------------------------------------
# Here we compare ClustSIGNAL cluster labels to annotations available with this 
# public data to assess its accuracy. We also assess the overall entropy 
# distribution of the samples in the dataset
spe2 |> colData() %>% 
  as.data.frame() %>%
  group_by(samples) %>%
  # group cells by samples and for cells in each sample 
  # calculate the following metrics
  summarise(ARI = aricode::ARI(Cell_class, ClustSIGNAL),
            NMI = aricode::NMI(Cell_class, ClustSIGNAL),
            min_Entropy = min(entropy),
            max_Entropy = max(entropy),
            mean_Entropy = mean(entropy))

## ----hypothal_spatialPlots----------------------------------------------------
# for plotting with scater R package, we need to add the spatial coordinates 
# to the reduced dimension section
reducedDim(spe2, "spatial") <- spatialCoords(spe2)

# spatial plot
spt_clust2 <- scater::plotReducedDim(spe2, colour_by = "ClustSIGNAL",
                                    # specify spatial low dimension
                                    dimred = "spatial", point_alpha = 1,
                                    point_size = 4, scattermore = TRUE) +
  scale_color_manual(values = colors) +
  facet_wrap(vars(spe2[[samples]]), scales = "free", nrow = 1) +
  guides(colour = guide_legend(title = "Clusters",
                               override.aes = list(size = 3))) +
  theme(text = element_text(size = 12))

# For cluster-level entropy distribution, we reorder the clusters by their 
# median entropy value in each sample
df_met2 <- spe2 |> colData() %>% as.data.frame()

box_clust2 <- list()
for (s in samplesList) {
  df_met_sub <- df_met2[df_met2[[samples]] == s, ]
  # calculating median entropy of each cluster in a sample
  ct_ent2 <- df_met_sub %>%
    group_by(as.character(ClustSIGNAL)) %>%
    summarise(mdEntropy = median(entropy))
  # reordering clusters by their median entropy
  # low to high median entropy
  cellOrder2 <- ct_ent2$mdEntropy
  names(cellOrder2) <- ct_ent2$`as.character(ClustSIGNAL)`
  cellOrder2 <- sort(cellOrder2)
  df_met_sub$ClustSIGNAL <- factor(df_met_sub$ClustSIGNAL, 
                                   levels = names(cellOrder2))

  # box plot of cluster entropy
  col_ent2 <- colors[as.numeric(names(cellOrder2))]
  box_clust2[[s]] <- df_met_sub %>%
    ggplot(aes(x = ClustSIGNAL, y = entropy, fill = ClustSIGNAL)) +
    geom_boxplot() +
    scale_fill_manual(values = col_ent2) +
    facet_wrap(vars(samples), nrow = 1) +
    labs(x = "ClustSIGNAL clusters", y = "Entropy") +
    ylim(0, NA) +
    theme_classic() +
    theme(strip.text = element_blank(),
          legend.position = "none",
          text = element_text(size = 12),
          axis.text.x = element_text(angle = 90, vjust = 0.5))
}

spt_clust2 / (patchwork::wrap_plots(box_clust2[1:3], nrow = 1) +
               plot_layout(axes = "collect")) +
  plot_layout(guides = "collect", heights = c(5, 3)) +
  plot_annotation(
    title = "Spatial (top) and entropy (bottom) distributions of clusters")

## ----hypothal_entropyPlots----------------------------------------------------
# we can assess the overall entropy distribution of each sample in the dataset
hst_ent2 <- spe2 |> colData() %>% 
  as.data.frame() %>%
  ggplot(aes(entropy)) +
  geom_histogram(binwidth = 0.05) +
  facet_wrap(vars(samples), nrow = 1) +
  labs(x = "Entropy", y = "Number of regions") +
  theme_classic() +
  theme(text = element_text(size = 12))

spt_ent2 <- scater::plotReducedDim(spe2, colour_by = "entropy",
                                  # specify spatial low dimension
                                  dimred = "spatial", point_alpha = 1,
                                  point_size = 4, scattermore = TRUE) +
  scale_colour_gradient2("Entropy", low = "grey", high = "blue") +
  scale_size_continuous(range = c(0, max(spe$entropy))) +
  facet_wrap(vars(spe2[[samples]]), scales = "free", nrow = 1) +
  theme(strip.text = element_blank(),
        text = element_text(size = 12))

hst_ent2 / spt_ent2 + plot_layout(heights = c(3,5)) +
    plot_annotation(
      title = "Entropy spread (top) and spatial distribution (bottom)")

## ----ClustSIGNALseq_data------------------------------------------------------
data(mEmbryo2) # this will load the logcounts and metadata to the environment

# as before, we read the data into a SpatialExperiment object
spe <- SpatialExperiment(assays = list(logcounts = me_expr),
                         colData = me_data, spatialCoordsNames = c("X", "Y"))

## ----ClustSIGNALseq_prep------------------------------------------------------
set.seed(100) # for reproducibility
# first we need to generate low dimension data for initial clustering
spe <- scater::runPCA(spe) 

## ----ClustSIGNALseq_step1-----------------------------------------------------
# The first step in the ClustSIGNAL algorithm is "initial clustering". 
# For this, we need only provide the spe object and low embedding information. 
# Other parameters have default values: batch = FALSE, batch_by = "None", 
# threads = 1, clustParams = list(clust_c = 0, subclust_c = 0, iter.max = 30, 
# k = 5, cluster.fun = "louvain")
spe <- clustSIGNAL::p1_clustering(spe, dimRed = "PCA")
# The cluster and subcluster labels are added to the spe object under cell 
# metadata
spe$nsCluster |> head() # clustering output
spe$initCluster |> head() # subclustering output (aka 'initial clusters' here)

## ----ClustSIGNALseq_step2-----------------------------------------------------
# The second step involves detecting the neighborhood of all cells. 
# We only need the spe object containing the initial clusters labels and sample 
# IDs for this. By default, ClustSIGNAL identifies NN = 30 nearest neighbors, 
# sorts the neighbourhood (sort = TRUE), and does not use parallel runs 
# (threads = 1)
outReg <- clustSIGNAL::neighbourDetect(spe, samples = "sample_id")
# This step generates a list of neighbourhood information.
outReg$nnCells[1:3, 1:3] # a neighborhood matrix with cell IDs
outReg$regXclust[[1]] # a list of arrays containing initial cluster proportions

## ----ClustSIGNALseq_step3-----------------------------------------------------
# The third step is to calculate entropy of each neighborhood. 
# We only need the spe object and initial cluster proportions for this step, and 
# it can run in parallel, by default with 1 core
spe <- clustSIGNAL::entropyMeasure(spe, outReg$regXclust)
# The entropy values are added to the spe object under cell metadata
spe$entropy |> head() # entropy values

## ----ClustSIGNALseq_step4-----------------------------------------------------
# The fourth step is to perform adaptive smoothing.
# This requires the spe object containing the entropy values and a neighborhood 
# matrix of cell IDs. Other parameters for which default values are provided 
# include number of neighbors (NN = 30), weight distribution type (kernel = 
# "G" for Gaussian), distribution spread (spread = 0.05 representing standard 
# deviation for Gaussian distribution), and number of cores (threads = 1) to use
# for parallel runs.
spe <- clustSIGNAL::adaptiveSmoothing(spe, outReg$nnCells)
# The adaptively smoothed gene expression data are added to the spe object under 
# assays
spe # 'smoothed' assay in the spe object

## ----ClustSIGNALseq_step5-----------------------------------------------------
# Finally, the fifth step involves performing clustering on the adaptively 
# smoothed data.
# We only need to provide the spe object containing the adaptively smoothed 
# data. This step has the same default parameters as the first step of initial 
# clustering. 
spe <- clustSIGNAL::p2_clustering(spe)
# The cluster labels are added to the spe object under cell metadata
spe$ClustSIGNAL |> head() # ClustSIGNAL cluster labels

## -----------------------------------------------------------------------------
sessionInfo()

